/*+-----------------------------------------------------------------------
	ecuicmd.c - ECU interactive command handler
	wht@n4hgf.Mt-Park.GA.US

  Defined functions:
	icmd(icmd_cmd)
	icmd_do_proc(ndoarg,doarg)
	search_cmd_list(cmd_list,cmd)

------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:08-25-1991-14:39-wht@n4hgf-SVR4 port thanks to aega84!lh */
/*:08-17-1991-18:29-wht@n4hgf-add kbdtest command */
/*:07-29-1991-20:56-wht@n4hgf-turn off memstat after frustrating evening */
/*:07-29-1991-17:57-wht@n4hgf-add memstat */
/*:07-25-1991-12:56-wht@n4hgf-ECU release 3.10 */
/*:05-21-1991-18:22-wht@n4hgf-add pushd/popd commands */
/*:05-21-1991-00:46-wht@n4hgf-added -3 error code to keyset_read */
/*:03-20-1991-05:25-root@n4hgf-experimental eto command */
/*:03-20-1991-04:55-root@n4hgf-Metro Link support + stat cmd changes */
/*:02-04-1991-19:03-wht@n4hgf-add multiscreen tag to stat */
/*:01-09-1991-22:31-wht@n4hgf-ISC port */
/*:12-24-1990-04:31-wht@n4hgf-experimental fasi driver command */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include "ecu.h"
#include "esd.h"
#include "ecufkey.h"
#include "patchlevel.h"

#define NEED_P_CMD
#include "ecucmd.h"

#if defined(FASI)
# include <local/fas.h>
#endif

#ifdef SVR4
# include <sys/termiox.h>
extern int hx_flag;
#endif

extern KDE keyset_table[];
extern char *makedate;
extern char curr_dir[CURR_DIRSIZ];
extern char hello_str[];
extern char keyset_name[];
extern char rcvr_log_file[];	/* if rcvr_log!= 0,log filename */
extern int current_ttymode;
extern long tty_escape_timeout;		/* timeout on waiting for char after ESC */
extern int tcap_COLS;
extern int tcap_LINES;
extern int tty_is_multiscreen;
extern int tty_is_ansi;
extern int tty_is_pty;
#if defined(METRLINK_X11R4_PTS)
extern int tty_is_pts;
#endif /* METRLINK_X11R4_PTS */
extern int errno;
extern int proc_level;
extern int proctrace;
extern int rcvr_log;			/* rcvr log active if != 0 */
extern int rcvr_log_append;	/* if true, append, else scratch */
extern int rcvr_log_raw;		/* if true, log all, else filter ctl chrs */
extern ulong colors_current;
extern FILE *plog_fp;
extern ESD  *plog_name;
extern char screen_dump_file_name[];

int protocol_log_packets = 0;
int want_bell_notify = 1;	/* want bell notify */

/*+-------------------------------------------------------------------------
	search_cmd_list(cmd_list,cmd)
returns -1 if cmd not found or insufficient chars for match
else token value for command
proc-only commands have 'mincnt' values 0
--------------------------------------------------------------------------*/
search_cmd_list(cmd_list,cmd)
register P_CMD *cmd_list;
register char *cmd;
{
	while(cmd_list->token != -1)
	{
		if(cmd_list->mincnt && minunique(cmd_list->cmd,cmd,cmd_list->mincnt))
			return(cmd_list->token);
		cmd_list++;
	}
	return(-1);

}	/* end of search_cmd_list */

/*+-------------------------------------------------------------------------
	icmd_do_proc(ndoarg,doarg)
--------------------------------------------------------------------------*/
icmd_do_proc(ndoarg,doarg)
int ndoarg;
char **doarg;
{
register erc;
ulong colors_at_entry = colors_current;

	kill_rcvr_process(SIGUSR1);
	ttymode(2);
	erc = do_proc(ndoarg,doarg);
	proc_file_reset();
	ttymode(1);
	sigint = 0;
	setcolor(colors_notify);
	ff(se,"[procedure finished]");
	setcolor(colors_at_entry);
	ff(se,"\r\n");
	start_rcvr_process(0);
	return(erc);
}	/* end of icmd_do_proc */

/*+-----------------------------------------------------------------------
	icmd(cmd)
 This function implements the built in commands
 It returns non-zero if program should terminate
------------------------------------------------------------------------*/
int
icmd(icmd_cmd)
register char *icmd_cmd;
{
#define ARG_MAX_QUAN 40
char *arg[ARG_MAX_QUAN];
char cmd[128];
int itmp,itmp2,itmp3;
int token;
int narg = 0;
ESD *tesd;
char s80[80];
char *epoch_secs_to_str();
long atol();
long ltmp;
long ltmp2;
char *find_procedure();
char *xon_status();

	icmd_history_add(icmd_cmd);		/* add to history list */
	strcpy(cmd,icmd_cmd);			/* get local copy of cmd string */
	switch(cmd[0]) 
	{
		case '.':		/* exit xmtr() (generally program too) */
			ff(se,"  disconnecting from line %s\r\n",shm->Lline);
			return(1);
		case '!':
		case '$':
		case '>':
			ff(se,"\r\n");
			shell(cmd);
			return(0);
		case '-':
			ff(se,"\r\n");
			exec_cmd(&cmd[1]);
			return(0);
		case '^':
			ff(se,"\r\n");
			phrase_help();
			return(0);
		case '?':
			icmd_help(0,(char **)0);
			return(0);
#ifdef DEBUG_SHM_SCREEN
		case '@':
			{
			FILE *fp = fopen("/tmp/screen","w");
			int y,x;
			char *cptr = (char *)shm->screen;
				for(y = 0; y < 43; y++)
				{
					for(x = 0; x < 80; x++)
						fputc(*cptr++,fp);
					fputc('\n',fp);
				}
				fclose(fp);
			}
			break;
#endif
		default:
			break;
	}

/* not single character argument */
	build_arg_array(cmd,arg,ARG_MAX_QUAN,&narg);

/* handle phrases */
	if(isdigit(*arg[0]))
	{
		phrases(narg,arg);
		return(0);
	}

/* search command list */
	if((token = search_cmd_list(icmd_cmds,arg[0])) < 0)
	{
		ff(se,"\r\n");
		if(find_procedure(arg[0]))
		{
			icmd_do_proc(narg,arg);
			return(0);
		}
		ff(se,"no such command or procedure ... HOME ? for help\r\n");
		return(0);
	}

	switch(token)
	{					/* keep alphabetized PLEASE */
		case CTrx:
		case CTry:
		case CTrz:
		case CTrk:
		case CTrs:
		receive_files_from_remote(narg,arg);
		break;

		case CTsx:
		case CTsy:
		case CTsz:
		case CTsk:
		case CTss:
		send_files_to_remote(narg,arg);
		break;

		case CTbaud:
		if(narg == 1)
			ff(se,"   baud rate is %u\r\n",shm->Lbaud);
		else
		{
			itmp = atoi(arg[1]);
			if(!lnew_baud_rate(itmp))
				ff(se,"   baud rate set to %u\r\n",shm->Lbaud);
			else
			{
				ff(se,"   invalid baud rate: %u\r\n",itmp);
				ff(se,"valid rates: 110,300,600,1200,2400,4800,9600,19200\r\n");
			}
		}
		break;

		case CTbreak:
		lbreak();
		ff(se,"   break sent\r\n");
		break;

		case CTcd:
		(void)change_directory(arg[1],(narg == 1) ? 0 : 1);
		break;

		case CTpushd:
		(void)push_directory(arg[1],(narg == 1) ? 0 : 1,0);
		break;

		case CTpopd:
		(void)pop_directory(arg[1],(narg == 1) ? 0 : 1,0);
		break;

		case CTclrx:
		lclear_xmtr_xoff();
		break;

		case CTdial:
		if(narg > 1)
		{
			ff(se,"\r\n");
			strcpy(shm->Llogical,arg[1]);
			if(!lookup_logical_telno())
				ff(se,"No such entry. Try 'dial' with no arguments\r\n");
			else
			{
				if(isdigit(shm->Llogical[0]) ||
					!find_procedure(shm->Llogical))
					DCE_dial();
				else
				{
					arg[0] = shm->Llogical;
					arg[1] = "!INTERACTIVE";
					icmd_do_proc(2,arg);
				}
			}
		}
		else
		{
			pde_list_manager();
		}
		break;

		case CTdo:
		ff(se,"\r\n");
		icmd_do_proc(narg - 1,&arg[1]);
		break;

		case CTptrace:
		if(narg > 1)
			proctrace = yes_or_no(arg[1]);
		ff(se,"  procedure trace %s",proctrace ? "on" : "off");
		if(proctrace > 1)
			ff(se," (%d)",proctrace);
		ff(se,"\r\n");
		break;

		case CTpcmd:
		itmp = strlen(arg[0]);
		if((tesd = esdalloc(256)) == (ESD *)0)
		{
			ff(se,"  no memory!!?!\r\n");
			break;
		}
		strcpy(tesd->pb,icmd_cmd + itmp + 1);
		tesd->cb = strlen(tesd->pb);
		ff(se,"\r\n");
		kill_rcvr_process(SIGUSR1);
		ttymode(2);
		if(itmp = execute_esd(tesd))
		{
			proc_error(itmp);
			esdshow(tesd,"");
		}
		esdfree(tesd);
		ttymode(1);
		start_rcvr_process(0);
		break;

		case CTplog:
		fputs("  ",se);
		if(narg > 1)
		{
			if(!strcmp(arg[1],"off"))
				plog_control((char *)0);
			else
				plog_control(arg[1]);
		}
		
		if(plog_fp)
			ff(se,"procedure logging: %s\r\n",plog_name->pb);
		else
			fputs("procedure logging off\r\n",se);
		break;

		case CTduplex:
		if(narg > 1)
		{
			switch(to_lower(*arg[1]))
			{
				case 'f': shm->Lfull_duplex = 1; ff(se,"  now "); break;
				case 'h': shm->Lfull_duplex = 0; ff(se,"  now "); break;
				default: ff(se,
"\r\nfirst argument character must be F or H for full or half duplex\r\n");
					break;
			}
		}
		else		/* no argument */
			ff(se,"  currently ");

		ff(se,"%s duplex\r\n",(shm->Lfull_duplex) ? "full" : "half");
		break;

		case CTexit:
		ff(se,"  disconnecting from line %s\r\n",shm->Lline);
		return(1);

		case CTfi:
		file_insert_to_line(narg,arg);
		break;

		case CThangup:
		ff(se,"  hanging up ...\r\n");
		DCE_hangup();
#if defined(FASI)
		{
			uchar msr = fasi_msr();
			ff(se,"hangup complete ... DCD is %s\r\n",
				(msr & MS_DCD_PRESENT) ? "STILL HIGH" : "low");
		}
#else
		ff(se,"hangup complete\r\n");
#endif
		break;

		case CThelp:
		icmd_help(narg,arg);
		break;

		case CTsdname:
		if(narg > 1)
		{
		char *new_file_name;

			itmp = 0;	/* do not need to free(new_file_name) */
			if(find_shell_chars(arg[1]))
			{

				if(expand_wildcard_list(arg[1],&new_file_name))
				{
					ff(se,"  %s\r\n",new_file_name);
					break;
				}
				itmp = 1;	/* need to free(new_file_name) */
			}
			else
				new_file_name = arg[1];

			screen_dump_file_name[0] = 0;
			if(*new_file_name != '/')
			{
				strcpy(screen_dump_file_name,curr_dir);
				strcat(screen_dump_file_name,"/");
			}
			strcat(screen_dump_file_name,arg[1]);
			if(itmp)
				free(new_file_name);
		}
		ff(se,"\r\nscreen dump name: %s\r\n",screen_dump_file_name);
		break;

		case CTlog:
		case CTloff:
		case CTllp:
		icmd_log(token,narg,arg);
		break;

		case CTnl:
		case CTnlin:
		case CTnlout:
		nlin_nlout_control(token,narg,arg);
		break;

		case CTparity:
		if(narg > 1)
		{
			switch(to_lower(*arg[1]))
			{
				case 'n':
					shm->Lparity = 0; break;
				case 'e':
				case 'o':
					shm->Lparity = to_lower(*arg[1]); break;
				default:
					ff(se,"   unrecognized parity: %c\r\n",*arg[1]);
					goto PARITY_DONE;
			}
			lset_parity(1);
		}
		ff(se,"   parity set to %c\r\n",
			(shm->Lparity) ? to_upper(shm->Lparity) : 'N');
PARITY_DONE: ;
		break;

		case CTpid:
		ff(se,": xmtr %d, rcvr: %d, parent: %d, group: %d\r\n",
			getpid(0),rcvr_pid,getppid(0),getpgrp(0));
		break;

		case CTpwd:
		fputs(" ",se);
		tcap_stand_out();
		ff(se," %s ",curr_dir);
		tcap_stand_end();
		fputs("\r\n",se);
		break;

		case CTredial:
		(void)DCE_redial(arg,narg);
		break;

		case CTrev:
		fputs("\r\n",se);
		fputs(hello_str,se);
		fputs("\r\n",se);
		ff(se,"%s\r\n",makedate);
#if defined(WHT) && !defined(NOMEMCHECK)
		_dump_malloc();
#endif
		break;

		case CTtime:
		get_tod(4,cmd);
		ff(se,": %s\r\n",cmd);
		break;

		case CTts:
		ff(se,"\r\n");
		sprintf(s80,"TTYIN %s (ttymode=%d)",get_ttyname(),current_ttymode);
		disp_line_termio(TTYIN,s80);
		ff(se,"\r\n");
		sprintf(s80,"comm line %s",shm->Lline);
		disp_line_termio(shm->Liofd,s80);
		break;

#if	defined(FASI)
		case CTfasi:
		icmd_fasi(narg,arg);
		break;
#endif
		case CTtty:
		ff(se,"   %s\r\n",get_ttyname());
		break;

		case CTda:
		case CToa:
		case CTxa:
		case CTax:
		icmd_conversion(token,narg,arg);
		break;

		case CTbn:
		if(narg > 1)
		{
			want_bell_notify = yes_or_no(arg[1]);
			shmx_set_rcvr_bn(want_bell_notify);
		}
		ff(se,"  bell notify %s%s\r\n",
					want_bell_notify ? "on" : "off",
					(want_bell_notify == 2) ? ": ring on incoming char" : "");
		break;

		case CTrtscts:
#if defined(RTSFLOW) || defined(SVR4) || defined(sun)
		if(narg > 1)
		{
			lRTSCTS_control(yes_or_no(arg[1]));
		}
		ff(se,"  RTS %s CTS %s\r\n",
#ifdef SVR4
			(hx_flag & RTSXOFF) ? "on" : "off",
			(hx_flag & CTSXON) ? "on" : "off");
#else
#if defined(RTSFLOW) /* SCO */
			(Ltermio.c_cflag & RTSFLOW) ? "on" : "off",
			(Ltermio.c_cflag & CTSFLOW) ? "on" : "off");
#else
#if defined(sun)
			"N/A",
			(Ltermio.c_cflag & CRTSCTS) ? "on" : "off");
#endif /* sun */
#endif /* RTSFLOW */
#endif /* SVR4 */
#else /* no RTS/CTS flow manipulation known to exist */
		ff(se,"  command not available\r\n");
#endif
		break;

		case CTeto:
		if(narg > 1)
		{
			itmp = atoi(arg[1]);
			if((itmp < 20) || (itmp > 1000))
			{
				ff(se," invalid\r\n");
			}
			tty_escape_timeout = (long)itmp;
			ff(se," set to");
		}
		ff(se," %ld msec\r\n",tty_escape_timeout);
		break;

		case CTnice:
		itmp = nice(0) + 20;
		if(narg > 1)
		{
			kill_rcvr_process(SIGUSR1);
			itmp2 = atoi(arg[1]);
			itmp3 = nice(-itmp + itmp2);
			ff(se," -> desired=%d, was %d, ",itmp2,itmp);
			if(itmp3 < 0)
				ff(se,"nice failed: %s\r\n",sys_errlist[errno]);
			else
				ff(se,"set to %d\r\n",itmp3);
			start_rcvr_process(0);
		}
		else
			ff(se," is %d\r\n",itmp);
		break;

		case CTfkey:
		if(narg < 2)
			keyset_display();
		else
		{
			switch(keyset_read(arg[1]))
			{
				case  0: keyset_display(); break;
				case -1: ff(se," cannot find ~/.ecu/keys\r\n"); break;
				case -2: ff(se," not found in ~/.ecu/keys\r\n"); break;
				case -3: ff(se," syntax error ... default restored\r\n");
				default:
					keyset_init();
					keyset_read("default");
					break; 
			}
		}
		break;

		case CTstat:
		ff(se,"\r\n\r\nDate/time");
		get_tod(4,cmd);
		ff(se,": %s\r\n",cmd);

		ff(se,"Console: %s %dx%d ",get_ttyname(),tcap_COLS,tcap_LINES);
		if(tty_is_ansi)
			ff(se,"<ansi> ");
		if(tty_is_multiscreen)
			ff(se,"<multiscreen> ");
		if(tty_is_pty)
			ff(se,"<pty> ");
#if defined(METRLINK_X11R4_PTS)
		if(tty_is_pts)
			ff(se,"<pts> ");
#endif /* METRLINK_X11R4_PTS */
		ff(se,"\r\nCommunications line: %s  ",shm->Lline);
		ff(se,"%u-%c-1\r\n",shm->Lbaud,
			(shm->Lparity) ? to_upper(shm->Lparity) : 'N');
		if(shm->Lmodem_off_hook)
		{
			ff(se,"Connected to %s: %s (%s)\r\n",
				shm->Lrname,shm->Ldescr,shm->Ltelno);
			ff(se,"at %s (elapsed %s)\r\n",
				epoch_secs_to_str(shm->Loff_hook_time,1,s80),
				get_elapsed_time(time((long *)0) - shm->Loff_hook_time));
		}
		ff(se,"Current directory: %s\r\n",curr_dir);
		ff(se,"Total chars transmitted: %ld",shm->xmit_chars);
		if(shm->Lmodem_off_hook)
			ff(se," (since CONNECT %ld)\r\n",
				shm->xmit_chars_this_connect);
		else
			fputs("\r\n",se);
		shmx_rc_report(&ltmp,&ltmp2);
		ff(se,"Total chars received:    %ld",ltmp);
		if(shm->Lmodem_off_hook)
			ff(se," (since CONNECT %ld)\r\n",ltmp2);
		else
			fputs("\r\n",se);
		if(keyset_name[0])
			ff(se,"Function key set '%s' loaded\r\n",keyset_name);
		else
			ff(se,"No function key set loaded\r\n");
		if(rcvr_log)
			ff(se,"Session log open: %s (%s mode)\r\n",
				rcvr_log_file,(rcvr_log_raw) ? "raw" : "filtered");
		else
			ff(se,"Session logging not active\r\n");
		ff(se,"Bell notify is %s\r\n",(want_bell_notify) ? "on" : "off");
		ff(se,"Duplex: %s\r\n",(shm->Lfull_duplex) ? "full" : "half");
		ff(se,"CR conversion:  incoming %s  outgoing %s\r\n",
			(shm->Ladd_nl_incoming) ? "CR/LF" : "CR",
			(shm->Ladd_nl_outgoing) ? "CR/LF" : "CR");
		ff(se,"keyboard ESC/funckey time constant = %ld msec\r\n",
			tty_escape_timeout);
		fputs("\r\n",se);
		break;

		case CTxon:
		if(narg > 1)
		{
			if(set_xon_xoff_by_arg(arg[1]))
				ff(se,"  argument error\r\n");
			else
				ff(se,"\r\n");
			break;
		}
		ff(se,"  xon/xoff flow control: %s\r\n",xon_status());
		break;

		case CTsgr:
		send_get_response(token,narg,arg);
		break;

/*
		case CTmkdir:
		if(narg < 2)
			ff(se,"  no argument\r\n");
		if(mkdir(arg[2],0755))
		{
			ff(se,"  ");
			pperror(arg[2]);
		}
		else
			ff(se,"  made directory %s\r\n",arg[2]);
		break;
*/

		case CTmemstat:
#if defined(MALLOC_3X)
	{
		struct mallinfo minfo;
		extern char *startbrk;
		char *sbrk();
		minfo = mallinfo();
		pputs("\n"); /* all this casting for 16- vs 32- bit ints */
		pprintf("%10lu total space in arena\n",(ulong)minfo.arena);
		pprintf("%10lu number of ordinary blocks\n",(ulong)minfo.ordblks);
		pprintf("%10lu number of small blocks\n",(ulong)minfo.smblks);
		pprintf("%10lu number of holding blocks\n",(ulong)minfo.hblks);
		pprintf("%10lu space in holding block headers\n",(ulong)minfo.hblkhd);
		pprintf("%10lu space in small blocks in use\n",(ulong)minfo.usmblks);
		pprintf("%10lu space in free small blocks\n",(ulong)minfo.fsmblks);
		pprintf("%10lu space in ordinary blocks in use\n",
			(ulong)minfo.uordblks);
		pprintf("%10lu space in free ordinary blocks\n",(ulong)minfo.fordblks);
		pprintf("%10lu cost of enabling keep option\n",(ulong)minfo.keepcost);
		pprintf("%08lx   startbrk\n",startbrk);
		pprintf("%08lx   sbrk(0)\n",sbrk(0));
		pprintf("%10lu break delta\n",sbrk(0) - startbrk);
	}
#else
		ff(se,"  not available\r\n");
#endif
		break;

		case CTkbdtest:
		ff(se,"\r\n");
		kbd_test();
		break;

		case 0:
		ff(se,"   procedure command not allowed in interactive mode\r\n");
		break;

		default:
		do_proc(narg,arg);
		sigint = 0;
		break;

	}
	return(0);		/* 0 == do not end program */

}	/* end of icmd */

/* end of ecuicmd.c */
/* vi: set tabstop=4 shiftwidth=4: */
