/*+-------------------------------------------------------------------------
	expresp.c - HDB expect/respond per SCO Devices file
	wht@n4hgf.Mt-Park.GA.US

 Meaning of some of the escape characters:
 \p - pause (approximately 1/4-1/2 second delay)
 \d - delay (2 seconds)
 \D - phone number/token
 \T - phone number with Dialcodes and character translation
 \N - null byte
 \K - insert a BREAK
 \E - turn on echo checking (for slow devices)
 \e - turn off echo checking
 \r - carriage return
 \c - no new-line
 \n - send new-line
 \nnn - send octal number
 \m### - sleep ### (decimal) milliseconds (non-standard)
 Speed - Hayes-specific speed-adaptive connect handler

  Defined functions:
	pcmd_expresp(param)
	execute_expresp(expresp_script)
	expect(str)
	respond(str)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:08-01-1991-05:00-wht@n4hgf-\n sent CR not NL */
/*:08-01-1991-04:31-wht@n4hgf-nap min of hzmsec if \m */
/*:08-01-1991-04:22-wht@n4hgf-detect NULL expect string */
/*:07-25-1991-12:57-wht@n4hgf-ECU release 3.10 */
/*:07-17-1991-07:04-wht@n4hgf-avoid SCO UNIX nap bug */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include "ecu.h"
#include "ecuerror.h"
#include "esd.h"
#include "var.h"
#include "proc.h"


#define MAX_FIELDS	50	/* max fields in a chat script */
#define MAX_EXPRESP	511	/* max length of a chat script */
#define MAX_EXPECT	63	/* max length of expect string */
#define DEFAULT_TIMEOUT_MSECS (10*1000L)

#define ERDEBUG(verb,str,arg) if(expresp_verbosity >= verb) \
	pprintf(str,arg)

long atol();

extern int proctrace;

int expresp_verbosity = 0;
ulong expect_timeout_msecs = DEFAULT_TIMEOUT_MSECS;
int expresp_echo_check = 0;

char last_Speed_result[32];

/*+-------------------------------------------------------------------------
	expect(str) - expect (read) string from remote
return code on failure, 0 on success
--------------------------------------------------------------------------*/
int
expect(str)
char *str;
{
	register char *cptr;
	char parsebuf[MAX_EXPECT + 1];
	int remaining = MAX_EXPECT;
	int erc;
	int old_ttymode = get_ttymode();
	register char *parsed = parsebuf;
	char s4[4];
	int itmp;
	long atol();

	if(!str)
	{
		ERDEBUG(2,"expect string NULL\n",0);
		return(eExpectRespondFail);
	}

	if(old_ttymode != 2)
		ttymode(2);

	if(!strncmp(str,"~t[",3) || !strncmp(str,"~m[",3))
	{
		expect_timeout_msecs = atol(str + 3);
		if(*(str + 1) == 't')
			expect_timeout_msecs *= 1000L;
		str += 3;
		ERDEBUG(2,"expect timeout = %lu msec\n",expect_timeout_msecs);
		if(cptr = strchr(str,']'))
		{
			str = cptr + 1;
			ERDEBUG(3,"expect: %s\n",str);
		}
		else
		{
			erc = eExpectRespondFail;
			goto RESTORE_TTYMODE_AND_RETURN_ERC;
		}
	}
	ERDEBUG(2,"expect: <<%s>>\n",str);
	if(!strlen(str) || !strcmp(str,"\"\""))
		goto GOT_EXPECTED;

	if(!strcmp(str,"Speed"))
	{
		LRWT lr;
		long ms_start;
		long ms_now;
		struct timeb now_timeb;
		ftime(&now_timeb);
		ms_start = (now_timeb.time * 1000) + now_timeb.millitm;
		do {
			last_Speed_result[0] = 0;
			lr.to1 = 90 * 100L;
			lr.to2 = 120L;
			/* allow interrupts + cooked read */
			lr.raw_flag = 0x80;
			lr.buffer = last_Speed_result;
			lr.bufsize = sizeof(last_Speed_result);
			lr.delim = "\n";
			lr.echo_flag = 0;
			lgets_timeout(&lr);
			ftime(&now_timeb);
			ms_now = (now_timeb.time * 1000) + now_timeb.millitm;
		} while(!sigint && !lr.count && ((ms_now - ms_start) < 90*1000L));

		if(sigint || strncmp(lr.buffer,"CONNECT",7))
			goto DID_NOT_GET_EXPECTED;
		else
			goto GOT_EXPECTED;
	}

	cptr = str;
	while(remaining && *cptr)
	{
		if(*cptr == '\\')
		{
			if(!*(++cptr))	/* if no character after escape, ... */
			{
				ERDEBUG(2," error: str ended with '\\'\n",0);
				goto DID_NOT_GET_EXPECTED;
			}

			if(isdigit(*cptr))	/* handle \ooo */
			{
				strncpy(s4,cptr,3);
				s4[3] = 0;
				sscanf(s4,"%o",&itmp);
				cptr += strspn(s4,"01234567");
				*parsed++ = (char)itmp;
				remaining--;
				continue;
			}

			switch(*cptr)
			{
			case 'n':
				*parsed++ = 0x0A;
				remaining--;
				break;
			case 'r':
				*parsed++ = 0x0D;
				remaining--;
				break;
			default:
				ERDEBUG(2," meaningless here: \\%c\n",*cptr);
				break;
			}
			cptr++;
		}
		else
		{
			*parsed++ = *cptr++;
			remaining--;
		}
	}
	*parsed = 0;

	if(!remaining)
		ERDEBUG(1," expect string too long\n",0);

	if(expresp_verbosity >= 3)
		hex_dump(parsebuf,strlen(parsebuf),"expecting",1);

	if(llookfor(parsebuf,expect_timeout_msecs,
		(expresp_verbosity < 3) ? expresp_verbosity : 0))
	{
GOT_EXPECTED:
		ERDEBUG(2,"[EXPECT SUCCEEDED]\n",0);
		erc = 0;
		goto RESTORE_TTYMODE_AND_RETURN_ERC;
		
	}

DID_NOT_GET_EXPECTED:
	ERDEBUG(2,"[EXPECT FAILED%s]\n",(sigint) ? " (interrupted)" : "");
	if(sigint)
	{
		sigint = 0;
		erc = eCONINT;
	}
	else
		erc = eExpectRespondFail;
	goto RESTORE_TTYMODE_AND_RETURN_ERC;

RESTORE_TTYMODE_AND_RETURN_ERC:
	if(old_ttymode != 2)
		ttymode(old_ttymode);
	return(erc);

}	/* end of expect */

/*+-------------------------------------------------------------------------
	respond(str) - send to remote

we enable SIGINT processing in here and return if 'sigint' detected,
but here, unlikme elsewhere, we do *not* reset sigint (se we do not
really "handle" it)
--------------------------------------------------------------------------*/
void
respond(str)
register char *str;
{
int itmp;
long ltmp;
char s4[4];
int send_no_cr = 0;
int old_ttymode = get_ttymode();

	if(sigint)
		return;
	ttymode(2);
	ERDEBUG(2,"respond: <<%s>>\n",str);
	while(*str)
	{
		if(*str == '\\')
		{
			/* handle \nnn - send octal number */
			if(isdigit(*++str))	/* handle \ooo */
			{
				strncpy(s4,str,3);
				s4[3] = 0;
				sscanf(s4,"%o",&itmp);
				str += strspn(s4,"01234567") - 1;	/* -1 cause str++ later */
				lputc((char)itmp);
			}
			else switch(*str)
			{
				case 'p':  /* pause (approximately 1/4-1/2 second delay) */
					ldraino(0);	/* wait for output to drain */
					if(Nap(400L) < 0)
						goto RETURN;
					break;
				case 'm':  /* nap a while (non-standard) */
					ltmp = atol(str + 1);
					str += strspn(str + 1,"0123456789");
					ldraino(0);	/* wait for output to drain */
					if(Nap((ltmp < hzmsec) ? hzmsec : ltmp) < 0)
						goto RETURN;
					break;
				case 'd':  /* delay (2 seconds) */
					ldraino(0);	/* wait for output to drain */
					if(Nap(2000L) < 0)
						goto RETURN;
					break;
				case 'D':  /* phone number/token */
					if(expresp_echo_check)
						lputs_paced(40,shm->Ltelno);	/* fake it */
					else
						lputs(shm->Ltelno);
					break;
				case 'T':  /* phnum with Dialcodes and char translation */
					if(expresp_echo_check)
						lputs_paced(40,shm->Ltelno);	/* fake it */
					else
						lputs(shm->Ltelno);
					break;
				case 'N':  /* null byte */
					lputc(0);
					break;
				case 'K':  /* insert a BREAK */
					lbreak();
					break;
				case 'E':  /* turn on echo checking (for slow devices) */
					expresp_echo_check = 1;
					break;
				case 'e':  /* turn off echo checking */
					expresp_echo_check = 0;
					break;
				case 'r':  /* carriage return */
					lputc(0x0D);
					break;
				case 'c':  /* no new-line */
					send_no_cr = 1;
					break;
				case 'n':  /* send new-line */
					lputc(0x0A);
					break;
			}

		}
		else
			lputc(*str);


		if(expresp_echo_check)
		{
			ldraino(1);		/* wait for output to drain, then flush input */
			Nap(40L);		/* fake it */
		}
		str++;
	}

	if(!send_no_cr)
		lputc(0x0D);

RETURN:
	ttymode(old_ttymode);

}	/* end of respond */

/*+-------------------------------------------------------------------------
	execute_expresp(expresp_script)

return 0 on success, else error code
--------------------------------------------------------------------------*/
int
execute_expresp(expresp_script)
char *expresp_script;
{
char *fields[MAX_FIELDS + 1];
int ifields;
int nfields;
int erc;
char expresp_copy[MAX_EXPRESP + 1];
char *expect_this;
char *send_on_fail;

#define EXPECT_STATE (!(ifields & 1))	/* even numbered fields are expect */
	expresp_echo_check = 0;
	last_Speed_result[0] = 0;

	ERDEBUG(2,"[EXPECT/RESPOND INITIAL TIMEOUT %ld MSEC]\n",
		expect_timeout_msecs);

	strncpy(expresp_copy,expresp_script,sizeof(expresp_copy));
	build_arg_array(expresp_copy,fields,MAX_FIELDS,&nfields);
	if(!nfields)	/* if no script, assume success */
	{
		ERDEBUG(2,"[EMPTY SCRIPT - EXPECT/RESPOND SUCCEEDED]\n",0);
		return(0);
	}

	for(ifields = 0; ifields < nfields; ifields++)
	{
		if(sigint)
			break;
		if(EXPECT_STATE)
		{
			expect_this = fields[ifields];
			while(1)	/* until break or return(error) */
			{
				if(send_on_fail = strchr(expect_this,'-'))
					*send_on_fail++ = 0;
				if(!(erc = expect(expect_this)))
					break;
				if((erc != eExpectRespondFail) || !send_on_fail)
				{
					ERDEBUG(2,"[EXPECT/RESPOND FAILED]\n",0);
					return(eExpectRespondFail);
				}
				if(expect_this = strchr(send_on_fail,'-'))
					*expect_this++ = 0;
				if(sigint)
					break;
				respond(send_on_fail);
			}
		}
		else
			respond(fields[ifields]);
	}
	if(sigint)
	{
		sigint = 0;
		ERDEBUG(2,"[CONSOLE INTERRUPT]\n",0);
		return(eCONINT);
	}
	ERDEBUG(2,"[EXPECT/RESPOND SUCCEEDED]\n",0);
	return(0);

}	/* end of execute_expresp */

/*+-------------------------------------------------------------------------
	pcmd_expresp(param)
expresp [-v[v...]] <exp-resp-str> [<timeout_msecs>]
--------------------------------------------------------------------------*/
int
pcmd_expresp(param)
ESD *param;
{
int erc;
int itmp;
char *cptr;
ESD *tesd;
char switches[8];

	if((tesd = esdalloc(MAX_EXPRESP + 1)) == (ESD *)0)
		return(eNoMemory);

	get_switches(param,switches,sizeof(switches));

	if(erc = gstr(param,tesd,0))
	{
		esdfree(tesd);
		return(erc);
	}

	expect_timeout_msecs = DEFAULT_TIMEOUT_MSECS;
	expresp_verbosity = (!!strchr(switches,'v')) || proctrace;
	if(expresp_verbosity)
	{
		cptr = switches,itmp = 0;
		while(*cptr)
			itmp += (*cptr++ == 'v');
		if(itmp > 1)
			expresp_verbosity = itmp;
	}
		
	if(erc = gint(param,&expect_timeout_msecs))
	{
		/* if something there non-integer */
		if(!end_of_cmd(param))
		{
			erc = eSyntaxError;
			goto RETURN;
		}
	}

	erc = execute_expresp(tesd->pb);

RETURN:
	esdfree(tesd);
	iv[0] = !!erc;
	if(proctrace)
		pprintf("$i00 = %7ld (0x%08lx,0%lo)\n",iv[0],iv[0],iv[0]);
	if(erc == eExpectRespondFail)
		erc = 0;
	return(erc);

}	/* end of pcmd_expresp */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of expresp.c */
