#if defined(SHARE_DEBUG)
#define LOG_UTMP
#endif
/*+-------------------------------------------------------------------------
	utmpstat.c - utmp status for XENIX/UNIX line
	wht@n4hgf.Mt-Park.GA.US

  Defined functions:
	strcmpi(s1,s2)
	utmp_status(line)

--------------------------------------------------------------------------*/
/*+:EDITS:*/
/*:08-25-1991-14:39-wht@n4hgf-SVR4 port thanks to aega84!lh */
/*:08-21-1991-02:23-wht@n4hgf-sun port */
/*:08-10-1991-17:39-wht@n4hgf-US_WEGOTIT handling */
/*:07-25-1991-12:59-wht@n4hgf-ECU release 3.10 */
/*:02-13-1991-02:00-ache@hq.demos.su-swap patch 5 US_ return values */
/*:02-07-1991-00:28-wht@n4hgf-utmp_status() was really messed up */
/*:02-03-1991-17:52-ache@hq.demos.su-fix for XENIX utmp handling bug */
/*:10-16-1990-20:43-wht@n4hgf-add SHARE_DEBUG */
/*:09-19-1990-19:36-wht@n4hgf-ecu_log_event now gets pid for log from caller */
/*:08-14-1990-20:40-wht@n4hgf-ecu3.00-flush old edit history */

#include "ecu.h"
#include "termecu.h"
#include "utmpstatus.h"
#include "ecuungetty.h"
#include "dialprog.h"
#include <errno.h>
#include <utmp.h>

#if defined(sun)
#define ut_id ut_host		//* fake debug info */
#else
#if !defined(ut_name)		/* nobody can keep their mind made up; ... */
#define ut_name ut_user		/* ... this is getting verry difficult, very old */
#endif
#endif /* sun */

#if defined(SVR4)
char *utmp_file = "/var/adm/utmp";
#else
char *utmp_file = "/etc/utmp";
#endif

struct utmp last_utmp;

/*+-------------------------------------------------------------------------
	strcmpi(s1,s2) - case-insensitive strcmp
In here rather than ecuutil.c since other executables besides ecu
uses this module and strcmpi needed there too 
--------------------------------------------------------------------------*/
int
strcmpi(s1,s2)
register char *s1;
register char *s2;
{
register char c1,c2;

	for( ; *s1 ; s1++,s2++)
	{
		if(isupper(*s1))
			c1 = tolower(*s1);
		else
			c1 = *s1;
		if(isupper(*s2))
			c2 = tolower(*s2);
		else
			c2 = *s2;
		if(c1 != c2)
			break;
	}
	return(*s1 - *s2);
}	/* end of strcmpi */

/*+-------------------------------------------------------------------------
	utmp_status(line) - check line status in utmp
'line' is "/dev/ttyxx"-style
returns US_ value and global utmp struct last_utmp;
--------------------------------------------------------------------------*/
int
utmp_status(line)
char *line;
{
register status = US_NOTFOUND;
#if !defined(sun)
register ufd;
int itmp;
#if defined(LOG_UTMP)
char logstr[128];
#endif

/*
 * crock/bozo alert: 
 * ut_name ain't but EIGHT characters long, but
 * EIGHT characters are often stored, so ya don't get no null
 * ut_id ain't but FOUR characters long, but
 * FOUR characters are routinely stored, so ya don't get no null
 */
char namecopy[sizeof(last_utmp.ut_name) + 1];
char idcopy[sizeof(last_utmp.ut_id) + 1];

	if((ufd = open(utmp_file,O_RDONLY,755)) < 0)
	{
		perror(utmp_file);
		termecu(TERMECU_LINE_OPEN_ERROR);
	}

	while((status == US_NOTFOUND) &&
		(read(ufd,(char *)&last_utmp,sizeof(last_utmp)) > 0))
	{
	    if(strcmpi(last_utmp.ut_line,line + 5))
			continue;

		strncpy(namecopy,last_utmp.ut_name,sizeof(last_utmp.ut_name));
		namecopy[sizeof(namecopy) - 1] = 0;

		strncpy(idcopy,last_utmp.ut_id,sizeof(last_utmp.ut_id));
		idcopy[sizeof(idcopy) - 1] = 0;

	    if(!strcmp(namecopy,"LOGIN"))
			status = US_LOGIN;
	    else if(!strcmp(namecopy,"DIALOUT"))
			status = (last_utmp.ut_pid,0 == xmtr_pid) ? US_WEGOTIT : US_DIALOUT;
	    else if((!strcmp(namecopy,"uugetty") || !strcmp(namecopy,"getty")))
		{
			if(itmp = line_locked(line))
			{
				if(itmp == LOPEN_WE_GOT_IT)
					status = US_WEGOTIT;
				else
					status = US_DIALOUT;
			}
			else
				status = US_LOGIN;
		}
		else if(!kill(last_utmp.ut_pid,0) || (errno != ESRCH))
			status = (last_utmp.ut_pid == xmtr_pid) ? US_WEGOTIT : US_LOGGEDIN;
	}

#if defined(LOG_UTMP)
	if(status == US_NOTFOUND)
		sprintf(logstr,"UTMP %s: no entry in utmp, status=%d",line,status);
	else
	{
	char *ctime();
		sprintf(logstr,"UTMP %s:%s:%s:%d",
		    namecopy,idcopy,last_utmp.ut_line,last_utmp.ut_pid,status);
	}
	ecu_log_event(getpid(),logstr);
#endif

	close(ufd);
#endif  /* sun */
	return(status);

}	/* end of utmp_status */

/* vi: set tabstop=4 shiftwidth=4: */
/* end of utmpstat.c */
